@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2018 Ben Collins <ben@cyphre.com>
+/* Copyright (C) 2015-2022 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
    This Source Code Form is subject to the terms of the Mozilla Public
@@ -527,7 +527,7 @@ static int jwt_verify_head(jwt_t *jwt)
 
 static int jwt_parse(jwt_t **jwt, const char *token, unsigned int *len)
 {
-	char *head = jwt_strdup(token);
+	char *head = NULL;
 	jwt_t *new = NULL;
 	char *body, *sig;
 	int ret = EINVAL;
@@ -536,6 +536,7 @@ static int jwt_parse(jwt_t **jwt, const char *token, unsigned int *len)
 		return EINVAL;
 
 	*jwt = NULL;
+	head = jwt_strdup(token);
 
 	if (!head)
 		return ENOMEM;
@@ -814,25 +815,6 @@ int jwt_del_grants(jwt_t *jwt, const char *grant)
 	return 0;
 }
 
-#ifdef _MSC_VER
-
-int jwt_del_grant(jwt_t *jwt, const char *grant);
-#pragma comment(linker, "/alternatename:jwt_del_grant=jwt_del_grants")
-
-#else
-
-#ifdef NO_WEAK_ALIASES
-int jwt_del_grant(jwt_t *jwt, const char *grant)
-{
-	return jwt_del_grants(jwt, grant);
-}
-#else
-int jwt_del_grant(jwt_t *jwt, const char *grant)
-	__attribute__ ((weak, alias ("jwt_del_grants")));
-#endif
-
-#endif /* _MSC_VER */
-
 const char *jwt_get_header(jwt_t *jwt, const char *header)
 {
 	if (!jwt || !header || !strlen(header)) {
@@ -1266,6 +1248,9 @@ int jwt_valid_new(jwt_valid_t **jwt_valid, jwt_alg_t alg)
 
 	(*jwt_valid)->status = JWT_VALIDATION_ERROR;
 
+	(*jwt_valid)->nbf_leeway = 0;
+	(*jwt_valid)->exp_leeway = 0;
+
 	(*jwt_valid)->req_grants = json_object();
 	if (!(*jwt_valid)->req_grants) {
 		jwt_freemem(*jwt_valid);
@@ -1294,6 +1279,22 @@ unsigned int jwt_valid_get_status(jwt_valid_t *jwt_valid)
 	return jwt_valid->status;
 }
 
+time_t jwt_valid_get_nbf_leeway(jwt_valid_t *jwt_valid)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	return jwt_valid->nbf_leeway;
+}
+
+time_t jwt_valid_get_exp_leeway(jwt_valid_t *jwt_valid)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	return jwt_valid->exp_leeway;
+}
+
 int jwt_valid_add_grant(jwt_valid_t *jwt_valid, const char *grant, const char *val)
 {
 	if (!jwt_valid || !grant || !strlen(grant) || !val)
@@ -1422,6 +1423,26 @@ int jwt_valid_set_now(jwt_valid_t *jwt_valid, const time_t now)
 	return 0;
 }
 
+int jwt_valid_set_nbf_leeway(jwt_valid_t *jwt_valid, const time_t nbf_leeway)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	jwt_valid->nbf_leeway = nbf_leeway;
+
+	return 0;
+}
+
+int jwt_valid_set_exp_leeway(jwt_valid_t *jwt_valid, const time_t exp_leeway)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	jwt_valid->exp_leeway = exp_leeway;
+
+	return 0;
+}
+
 int jwt_valid_set_headers(jwt_valid_t *jwt_valid, int hdr)
 {
 	if (!jwt_valid)
@@ -1472,12 +1493,12 @@ unsigned int jwt_validate(jwt_t *jwt, jwt_valid_t *jwt_valid)
 
 	/* Validate expires */
 	t = get_js_int(jwt->grants, "exp");
-	if (jwt_valid->now && t != -1 && jwt_valid->now >= t)
+	if (jwt_valid->now && t != -1 && jwt_valid->now - jwt_valid->exp_leeway >= t)
 		jwt_valid->status |= JWT_VALIDATION_EXPIRED;
 
 	/* Validate not-before */
 	t = get_js_int(jwt->grants, "nbf");
-	if (jwt_valid->now && t != -1 && jwt_valid->now < t)
+	if (jwt_valid->now && t != -1 && jwt_valid->now + jwt_valid->nbf_leeway < t)
 		jwt_valid->status |= JWT_VALIDATION_TOO_NEW;
 
 	/* Validate replicated issuer */
