@@ -34,51 +34,51 @@ static unsigned char key[16384];
 static size_t key_len;
 
 static const char jwt_rs256_2048[] = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.ey"
-	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLmN5cGhyZS5jb20iLCJyZWYiOiJYWF"
-	"hYLVlZWVktWlpaWi1BQUFBLUNDQ0MiLCJzdWIiOiJ1c2VyMCJ9.cl9YHrbydUPb8dbij"
-	"SZzpTa-r-Z2bFz8r1DEQeqGB2ncHlNvYRLa3wa-IbOSQGPVok9xMutxc2ngm0cvquOOW"
-	"WVZIpYz3IdZQaCZ4G2PtTwnmhblSnqB-1ZvbUljBHjIoeXDTq2Msph2sjED9YKHKcjIm"
-	"kwil1cp75bnZMoKW3kDuNdq1vUwZDLdE_YRMpA53sTsoXHNSBzQwrIFEdCA8OA2rS-9R"
-	"IYtbLnKUZH4GXe2wb5y7pB21qqIdSl9k7yuD90k7LaCQDNLvrI1_cQB9wQcqqFA0qFc2"
-	"UxbiRRsC65eRZ1PfdZ8I_scukh5Vts5PNaRdE-_y_bpZKPaUu-WwA";
+	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLm1hY2xhcmEtbGxjLmNvbSIsInJlZi"
+	"I6IlhYWFgtWVlZWS1aWlpaLUFBQUEtQ0NDQyIsInN1YiI6InVzZXIwIn0.RlJPQst_lp"
+	"MJUsbnzlT2Mf3xzlHyUlVaQM_PJ1_vpBf1gHkhv-0hm3pa1_HRvpqg5UdDF3iOMLT0GU"
+	"j3W8JveaSvXKFeZdRpQGqmC7MZ7NzaYtyaDT7asniIVDf0JomD8Cfq8IdOn2ZREpbuJ6"
+	"moPwwvJ2zwL3vY-7w5A7ZQ3fxUedPuzn9n6tbEnuXcbDMyWQjen5poYmmvoIrDbzK0Zb"
+	"KbAJ5VrJwME_fZnPHS4c3b8rZGdBJCPI8oT2On6a9LrVqY3riqqHeiSqewfjDsox4tL2"
+	"G5KUpqK0oJmnZPGTnNY774PGabpcPBNbfMJqi8o8r0a7pa7sy6B59P7slUdw";
 
 static const char jwt_rs384_4096[] = "eyJhbGciOiJSUzM4NCIsInR5cCI6IkpXVCJ9.ey"
-	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLmN5cGhyZS5jb20iLCJyZWYiOiJYWF"
-	"hYLVlZWVktWlpaWi1BQUFBLUNDQ0MiLCJzdWIiOiJ1c2VyMCJ9.W0hvRx5yJaR3serc6"
-	"FoXy-0XeKxekQ3qIcIiBE4mTlH_E9vjO3Be_TAu2HTu4Th81VWzr6F5hbB2hGPP9UrCn"
-	"gwVx-mmRDQBEzdpS2n_Quews9GE68VdVP6zoVeQnOx4Htyg8eobmFfxxiqEegLJNqyTb"
-	"CJyZ7XeEDtLpa1iZBz__wlU5gHSinxogTsyfd4N5eRPD6_hjG6iGjQ2i6KmEhiR5SKYI"
-	"FLIfMm872Va4TmgxRQjL-mO-4xpo5ySgpWHuRvN6rAE6tGGTswiQoQq3zQbLCF1DScHv"
-	"owUbGaAHX46ACNbDHe1N8JbvVUyM0vrgKbmZ4Nlg5kf_b3MMYCQ_zlL2msTr-8NrU0Gc"
-	"5g1f1IL2GVPEZPxQT0Z5c9mRGIBN82OyrCftfy3O-PRWU0yLM5Odt_QzL7KfIIYLZfws"
-	"ADkqm3_-bst60DfTWWIr63smo1XAeLs1sqxO3QFxFHyaYcHl17QvX-2vxGJACzbjvGN4"
-	"9MeACklEyDVcWOQyNPu5kgPGhyjmH_A8qlcKGka8OqYnNIOSj54GTf5gWObBrPIXN8mL"
-	"7e64iI4dtlaA9-9KWLjpCLYlFGVMntu1YgzF9-xQ0RFxvcaHkXNVSQs61eMH-LH7kn2f"
-	"cfFKjDDFhm7OHNfCNcgTzfCDKIfOgh1FPAa7gRIBauutr-9n-TW8TY";
+	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLm1hY2xhcmEtbGxjLmNvbSIsInJlZi"
+	"I6IlhYWFgtWVlZWS1aWlpaLUFBQUEtQ0NDQyIsInN1YiI6InVzZXIwIn0.E9e6Chv39H"
+	"FQqhfUFcCxUzyS6yHQvBex_OaQNIPqeoPR_FDquiLhQUJj45IGd8_aVT8DvfzTFSHOP1"
+	"cP_UbzCYtZ8VC0o-idxyjTnlnqMJy75xNBzPUy6OSpTeX9yVQCtIkT6kIom4j35ABibk"
+	"_MdCVqnWG2fijDEeTlLD7uDRcCTGckqQhVOI3t4iIytiUVPnboFiaPLfei_mPcJ6CiOI"
+	"QYF4VOWlDllgUFrQ0M0nKm4Pq6bVaBIMzF0hJrPn_7GCV3XmVLthcObljfydaNm_CcIY"
+	"g7y_8OT_8yAvDlbKBe5jVeq-7_lLCinarkGUZ5ryA2lbC6yPBgtaAU3g6XxP60n6To7z"
+	"akV_5dgcPJFDlTkoBI6pPH3Zf50UYsf-wR4D2J3fP4rcYco4HGtxZ89tfoNYCB8z5-GA"
+	"ImSJRbVSsdadwSKKgldlG9XR13Cq-Ox8Hc7qCd1tTC-NeY31XMuWxd9981bQMeGhBkyJ"
+	"fFnIksh8xyyO0uOPvgmchOtG8bSImfZZaeBI6TcPJ89Oo3iD6NmPdO5AUqv6NB7Y46zH"
+	"GUXCeTuumVk7I_PJ9laICW4-cx1zHDvPY3TphVVTudSqWdUDMjwhrRI23569DByMJE9J"
+	"XpTg7HV_17EPTDWExc6TQVkmmY4QbS0pVbKyJTwwKmnu2F9o2fl1N0NOw1SOM";
 
 static const char jwt_rs512_8192[] = "eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.ey"
-	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLmN5cGhyZS5jb20iLCJyZWYiOiJYWF"
-	"hYLVlZWVktWlpaWi1BQUFBLUNDQ0MiLCJzdWIiOiJ1c2VyMCJ9.Lon6Vh8pjAXq_XJfE"
-	"r_0ppYuawbL-AYqjw0BQep0Dc5hU-qOMHjZslnvS8MMkdzvsjvr10O0HHive8cw5XMoj"
-	"LndSXeKejzsLdYoU93cpmz-Eb_AX8XcWx201JlF8lDZYest1__yplNiSwrcfgsef6Upi"
-	"07P8hjHjQqF2W-n037ckgJANQ4wcacSUEcxtjY1Sk5Y-FMsckx-lD60hCYYxcMUt7Uoz"
-	"qXlFpk28H1xx7saVlheJGsgZYX2bVNW6PhT_VKjIwOP3MWBxndvlQfPG8ZJRWE2puCum"
-	"MAV7ncs6JcIrOstXx4RtXxgwbbjXta114np7sxIMnA8pYe6q_uTstFIJYry0YkBfZ9rO"
-	"PjHibREMOf7iFOwlls7bn_XwcBaYBEoo5R4yP8F_HVdr17gw6iorzA0iqnn_79WtQY2Z"
-	"THOFU8v2NxHat_MvuIWy79v05NY852wMjuxA_Z7nd1DP5xv5YYOV1AKG9g1UrsdAyMK1"
-	"75z75jy_HLmx_Y51zmDKwuovHMMZvtDOBtpM6nPVx0kTQkf6ngtpHGDtMxAd4SXmoUg8"
-	"76zQ04p64Wgn0m2_cvr2D35j-4ddAiEtWGEVygEEcA7amyhh56yuM8K6bxAguZM_81Iq"
-	"OuCJFD4qW6KtmYOFTpDPOnJuvVJLxHz7nixH8U-q5UcSW08ScbQq3XPjtoCAG6LKh94S"
-	"q663kRjA8a6RpeC1GHFRM1Jit2LgGVY8YzxA0d6vXOB9gTKTmuvhvHKtWCRhCx-SorC0"
-	"Zkn_910ZzwF3gVdbejvyxDfLa8eqLAr48SsBUtU-l69tu1X8tNJ6Ug549Y1yg_-BOzPz"
-	"Zb4r3tFW4lV_zTVyNGxDp4bq8LoG9aMmcB04r-f6hnwe9hlA-6pBaZ0PD6eYHFpAXJAi"
-	"XJc22VGZpI0f7G4BywOqEJ5qq5aivppxdLRQQQiHUk-_56zxx5q70301PH7rgf00Uuwa"
-	"zZI00rQAB5XhOKqIXuGEYnL9-AFKLl0Gipr_kMqVPPZ1PgVTcDp1ZuLub_iC7_Fh4_aV"
-	"lZL-Tx80Y0aRfiDiLGYtSlqiVYnASCqeyzs_10Bext7A0MjTiVd49h0Ek6vfEymJaRtL"
-	"tomuPM1UTZEePW4sx-0ewW5D2OGP2sGjm56bM4LQfXlGo3sQJBzc8jG1JOLXtZ0S6QqV"
-	"PIy02kuySuWXhUkfLKy7nCj1XFPohkrh32hef2Iml-ATnQzshIL2Ev7BL53W64XVaXsr"
-	"qUNa4kV6TG20X7OvLQW81vdd8hIJIUq-xKNlQw25fMQpCy4matYx-FKZ-LCtigkzVFXg"
-	"24qGD3tFIsOSuZUmEjYu1Sbn6FfO9AuPUnC-qwVwloNPPCAjP2bqUF9OA";
+	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLm1hY2xhcmEtbGxjLmNvbSIsInJlZi"
+	"I6IlhYWFgtWVlZWS1aWlpaLUFBQUEtQ0NDQyIsInN1YiI6InVzZXIwIn0.F3_fF6RGJy"
+	"B_apyq_ljxLP7luYC78wT23pPKAyRSMiDi0g-7Yfohv5_p0rBOCsT1H-1_rImWiIcsdJ"
+	"87oYfQw95G_pHcK56ag7_a1i-jQCV7ZyRAuesDcM1YRealKvdnof08Jw392h685XNK2l"
+	"mqvZ436Hz4oCcQimwuKeR0ndAgm38y-_FswOlb3POwHBJcDtDR3UTvCONyqgcCD5hl5J"
+	"edjA8GA0Mvlp4JV2-ctsNYaDOMQRw3S8hJOXvKySQsbckEj8pC_bCrcbSR8BUJBrjM5Q"
+	"SLslcYGJopyCYAApGsq3t4-uY3Dx-QjWBMgSIgD9BONjv4VXvsfrllWUQmK88qJ4WQFO"
+	"L0PUSJy57lZjpjXReafJcJgdrGI-tFHgqkcb1VxmqnnIFRqadgjJ1BkfFvdExJiOgnvN"
+	"SEQ16AYy2FlxZeaabqyOjc8IlGi7Z9hYZgImL_qn0REJXgvxNtabA9-A4cEMWVDtYkCn"
+	"F_VyYeMOMYKMuoFW5ubLfOhYhDnd_yvkrLzZ76BGXpHtg7cfpnN8dNWp8irwjSByBH4H"
+	"lB0F9NFZydNB8wylFsKesNhmkQYcLnchhA4dBZV888NKVfqIBcM-GWXExtmON0SXg_HX"
+	"YmbO6zTgQ4tGAk4HglRWJYlfgORdcBBtnlwUHXm5L7_0J8KHoyNUGbA2XU1krVhBN-A7"
+	"sxtLof6MZP8c7p65Oc6DH8FulpmnBt8yVID2GnXPfcLs-RgM8QnlSGbNhn9WT2kmBTIV"
+	"eniVfp_IJ40Kd4SROGa_XbXMTufympqyYSSxtqHblGgtolVGdpN05FDvInG0J0dojYxT"
+	"_puZ-fvBAMXRPeoC_t_1ScfER4CjsUedbReQLZmE-9nJKhCZKqiba5qCbq8riZYiROGJ"
+	"rtVryTywLzw7XX4D-s9oJsEk6ELSRI-buXuCqyCmbdRpFz-i2-VPmNQIalk_0Pq_dOZN"
+	"y0GCvcezkhx1quGKPDmFskJnvKZmC70er0DvOQl1A909kFivxIXfTzlu8jUJt8PPi0gU"
+	"-nnOGSYxC8tD16vwHvAP3KPYvUzmC2N0r6_yM2_Y-JH5Vypeecbuh66cx18Bqk1nQYfg"
+	"BLjuJwQSIKRNNBLtgU7mcyI0Lj4-TWbE-22dYYvKcPMxSmfwAmJUm7ZFUAq_Ok-46AmV"
+	"RYg-h7bZZlfutOiWuoBrmnqQ6dEDGjXiEgWhtAx5HG3qn1_vmA3JQxJAWEfuhHa3IWac"
+	"MDRrJehImeyDE0H0rpOsxSXOjnDqiFBsf9d0-zJNFvo9tWlK_-d-N40BIy5eZm37FKG7"
+	"g2rFmXtuicUs6jiwu0_tHSi1fPKO7YN2ezQc9HAoBvvrur1z_XGbDSmFTQNTv0Cg";
 
 static const char jwt_rs256_invalid[] = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.ey"
 	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLmN5cGhyZS5jb20iLCJyZWYiOiJYWF"
@@ -96,7 +96,7 @@ static void read_key(const char *key_file)
 	fp = fopen(key_path, "r");
 	ck_assert_ptr_ne(fp, NULL);
 
-	free(key_path);
+	jwt_free_str(key_path);
 
 	key_len = fread(key, 1, sizeof(key), fp);
 	ck_assert_int_ne(key_len, 0);
@@ -119,7 +119,7 @@ static void __test_alg_key(const char *key_file, const char *jwt_str,
 
 	read_key(key_file);
 
-	ret = jwt_add_grant(jwt, "iss", "files.cyphre.com");
+	ret = jwt_add_grant(jwt, "iss", "files.maclara-llc.com");
 	ck_assert_int_eq(ret, 0);
 
 	ret = jwt_add_grant(jwt, "sub", "user0");
@@ -139,7 +139,7 @@ static void __test_alg_key(const char *key_file, const char *jwt_str,
 
 	ck_assert_str_eq(out, jwt_str);
 
-	free(out);
+	jwt_free_str(out);
 	jwt_free(jwt);
 }
 
@@ -155,8 +155,12 @@ static void __verify_alg_key(const char *key_file, const char *jwt_str,
 	ck_assert_int_eq(ret, 0);
 	ck_assert(jwt != NULL);
 
-	ck_assert(jwt_get_alg(jwt) == alg);
+	jwt_valid_t *jwt_valid = NULL;
+	jwt_valid_new(&jwt_valid, alg);
 
+	ck_assert_int_eq(JWT_VALIDATION_SUCCESS, jwt_validate(jwt, jwt_valid));
+
+	jwt_valid_free(jwt_valid);
 	jwt_free(jwt);
 }
 
@@ -172,6 +176,34 @@ START_TEST(test_jwt_verify_rs256)
 }
 END_TEST
 
+START_TEST(test_jwt_validate_rs256)
+{
+	jwt_t *jwt = NULL;
+	jwt_valid_t *jwt_valid = NULL;
+	int ret = 0;
+
+	read_key("rsa_key_2048-pub.pem");
+
+	ret = jwt_decode(&jwt, jwt_rs256_2048, key, key_len);
+	ck_assert_int_eq(ret, 0);
+	ck_assert(jwt != NULL);
+
+	jwt_valid_new(&jwt_valid, JWT_ALG_RS256);
+	ck_assert(jwt_valid != NULL);
+
+	ret = jwt_valid_add_grant(jwt_valid, "iss", "files.maclara-llc.com");
+	ck_assert_int_eq(ret, 0);
+
+	ret = jwt_valid_add_grant_int(jwt_valid, "iat", TS_CONST);
+	ck_assert_int_eq(ret, 0);
+
+	ck_assert_int_eq(JWT_VALIDATION_SUCCESS, jwt_validate(jwt, jwt_valid));
+
+	jwt_valid_free(jwt_valid);
+	jwt_free(jwt);
+}
+END_TEST
+
 START_TEST(test_jwt_encode_rs384)
 {
 	__test_alg_key("rsa_key_4096.pem", jwt_rs384_4096, JWT_ALG_RS384);
@@ -227,7 +259,7 @@ START_TEST(test_jwt_encode_rsa_with_ec)
 
 	read_key("ec_key_secp384r1.pem");
 
-	ret = jwt_add_grant(jwt, "iss", "files.cyphre.com");
+	ret = jwt_add_grant(jwt, "iss", "files.maclara-llc.com");
 	ck_assert_int_eq(ret, 0);
 
 	ret = jwt_add_grant(jwt, "sub", "user0");
@@ -312,7 +344,7 @@ START_TEST(test_jwt_encode_invalid_key)
 
 	read_key("rsa_key_invalid.pem");
 
-	ret = jwt_add_grant(jwt, "iss", "files.cyphre.com");
+	ret = jwt_add_grant(jwt, "iss", "files.maclara-llc.com");
 	ck_assert_int_eq(ret, 0);
 
 	ret = jwt_add_grant(jwt, "sub", "user0");
@@ -345,6 +377,7 @@ static Suite *libjwt_suite(void)
 
 	tcase_add_test(tc_core, test_jwt_encode_rs256);
 	tcase_add_test(tc_core, test_jwt_verify_rs256);
+	tcase_add_test(tc_core, test_jwt_validate_rs256);
 	tcase_add_test(tc_core, test_jwt_encode_rs384);
 	tcase_add_test(tc_core, test_jwt_verify_rs384);
 	tcase_add_test(tc_core, test_jwt_encode_rs512);
@@ -357,7 +390,7 @@ static Suite *libjwt_suite(void)
 	tcase_add_test(tc_core, test_jwt_verify_invalid_cert_file);
 	tcase_add_test(tc_core, test_jwt_encode_invalid_key);
 
-	tcase_set_timeout(tc_core, 60);
+	tcase_set_timeout(tc_core, 120);
 
 	suite_add_tcase(s, tc_core);
 
